/*
  /-------------------------------------------------\
  |                                                 |
  |                                                 |
  |     /=====\     /=====\     |\          /|      |
  |    /       \   /       \    | \        / |      |
  |    |           |            |  \      /  |      |
  |    |           |            |   \    /   |      |
  |    \       /   \       /    |    \  /    |      |
  |     \=====/     \=====/     |     \/     |      |
  |                                                 |
  |                                                 |
  |           =====>>>>> 2002 <<<<=====             |
  |                                                 |
  \-------------------------------------------------/
  
  ===================================================
  
  Name:       ccm.C
  
  @Copyright: Damian J. J. Farnell
  
  Date:       Wed Sep 25 15:32:06 BST 2002
  
  Purpose:    Determine and solve CCM equations.
  
  ===================================================
  
  ---------------------------------------------------

  MODULE NAME: CCM.C 

  PURPOSE:     MAIN ROUTINE WHICH CALLS EACH MODULE

  ---------------------------------------------------

*/

// #include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "../include/nr.h"
#include "../include/nrutil.h"
#include "../include/Constants.h"
#include "../include/GlobalVariables.h"
#include "../include/FunctionPrototypes.h"

#if (PARALLEL_FLAG)
#include<mpi.h>
#endif

// Run in parallel with:

// mpirun -n 1 -p ./ccm.x 

main(int argc, char *argv[])
{
  
  int i;
  int run_option;
  time_t  t1, t2;   /* measure speed */
  time(&t1);        /* get unix time in seconds */

#if (PARALLEL_FLAG)

  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD,&npe);
  MPI_Comm_rank(MPI_COMM_WORLD,&myid);

  if (npe > 16) {
    printf("Something wrong in ccm.C, npe > 16...\n"
           "Exiting!!\n\n");
    exit(1);
  }
  cout << "NPE = "<< npe << " MYID = " << myid << endl;

#else

  npe=1;
  myid=0;

#endif

  /* ccm.C: 

     Purpose: To enumerate all CCM spin cluster configurations within 
     an LSUBm (or other) approach and then determine and solve the
     CCM ground ket- and bra-state equations.

     ====> The Code Is Split Into 5 Modules: <====

     1) Input.C

     ..(a) Read In Control Variables and Initialise Data.

     ..(b) Define the crystallographic lattice.

     2) Define.C

     ..(a) Enumerate the connected clusters.

     ..(b) Determine the unconnected clsuters.

     ..(c) Choose a subset of these clusters (e.g., those
     ......which conserve s_T^z).

     3) Obtain.C

     ..(a) Loop over total number of fundamental configs.
     
     ..(b) Determine the CCM equations for each `term.'
  
     ..(c) Save CCM equations to disk.

     4. Collect.C

     ..(a) Collects like terms together and thus reduces disk usage.

     5. Solve.C

     ..(a) Solve ket- and bra-state equations.
     ..(b) Find ground-state energy, sublattice magnetisation, etc.

  */

  if (argc == 3) {
    run_option=atoi(argv[2]);
    if (myid==0)
      printf("Run option is =%d\n",run_option);
  } else {
    printf("Input script name and run option...\nExiting!!\n\n");
    exit(1);
  }

  fflush(stdout);
  /* dont buffer output to get all outputs immediately */
  setvbuf(stdout,NULL,_IONBF,0);
  /* be sure all threads have runoption set */
#if (PARALLEL_FLAG) 
  MPI_Barrier(MPI_COMM_WORLD);
#endif

  input(argc,argv);

#if (PARALLEL_FLAG) 
  MPI_Barrier(MPI_COMM_WORLD);
#endif

  if (run_option == 1 || run_option == 4) 
    define();

#if (PARALLEL_FLAG) 
  MPI_Barrier(MPI_COMM_WORLD);
#endif
  time(&t2);        /* get unix time in seconds */
  printf("Time: define finished after %ds\n",t2-t1);

  if (run_option == 2 || run_option == 4) {
    obtain();  /* get and store collected CCM terms to data.input */
  }
  
#if (PARALLEL_FLAG) 
  MPI_Barrier(MPI_COMM_WORLD);
#endif
  time(&t2);        /* get unix time in seconds */
  printf("Time: obtain finished after %ds\n",t2-t1);

  if (run_option == 3 || run_option == 4) 
    solve();   /* read data.input (often) and output data.output */
  
#if (PARALLEL_FLAG)
  MPI_Finalize();
#endif
  time(&t2);        /* get unix time in seconds */
  printf("Time: solve finished after %ds\n",t2-t1);

  printf("\nPress a key to close this window.\n\n");
  //  getchar();

}


